<?php

/**
 * Builds the output for the domain list in the DRD dashboard including
 * filters, operations and the list of domains with their details.
 *
 * @param type $sid
 * @return type
 */
function drd_domain($sid = 0) {
  if ($sid) {
    $_SESSION['drd']['filter']['domain'] = array();
    if ($sid == 'reset') {
      // Nothing else to do, the filter is reset already
    }
    else {
      $_SESSION['drd']['filter']['domain']['sids'] = array($sid);
    }
    drupal_goto('admin/config/system/drd/dashboard/domains');
  }
  if (isset($_POST['op'])) {
    if ($_POST['op'] == t('Reset')) {
      $_SESSION['drd']['filter']['domain'] = array();
      drupal_goto('admin/config/system/drd/dashboard/domains');
    }
    if ($_POST['op'] == t('Execute')) {
      if (empty($_POST['operation'])) {
        drupal_set_message('No operation selected.', 'warning');
      }
      else if (empty($_POST['domains'])) {
        drupal_set_message('No domain selected.', 'warning');
      }
      else {
        drd_operation('domain', $_POST['domains'], $_POST['operation']);
      }
    }
  }
  $build['messages_placeholder'] = array(
    '#markup' => '<div id="drd-messages"><span class="element-hidden">' . t('Click here to delete') . '</span><div class="content"></div></div>',
  );
  $build['domain_filter_form'] = drupal_get_form('drd_domain_filter_form');
  $build['domain_list'] = drupal_get_form('drd_domain_list');

  return $build;
}

/**
 * Builds the domain list according to the current filter settings.
 *
 * @param type $form
 * @param type $form_state
 * @return string
 */
function drd_domain_list($form, &$form_state) {
  $header = array(
    'name'   => array('data' => t('Name'),      'field' => 'd.name', 'sort' => 'asc'),
    'url'    => array('data' => t('URL'),       'field' => 'd.url'),
    'usessl' => array('data' => t('SSL'),       'field' => 'd.usessl'),
    'status' => array('data' => t('Status'),    'field' => 'd.status'),
    'inst'   => array('data' => t('Installed'), 'field' => 'd.installed'),
    'aliases'=> array('data' => t('Aliases')),
    'heartbeat'=> array('data' => t('Heartbeat') . '<span class="drd-heartbeat-toggle pointer off">' . drd_icon('toggle', 'power') . '</span>'),
  );

  $query = db_select('drd_domain', 'd');

  drd_domain_build_filter_query($query);

  $count_query = clone $query;
  $count_query->addExpression('COUNT(d.did)');

  $query = $query->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('d')
    ->limit(variable_get('drd_items_in_lists', 25))
    ->orderByHeader($header)
    ->setCountQuery($count_query);
  $result = $query->execute();

  if ($result->rowCount() > 0) {
    drd_operation_build_form($form, 'domain');
  }

  $domains = array();
  foreach ($result as $d) {
    $domains[$d->did] = drd_domain_build_row($d);
  }

  $form['domains'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $domains,
    '#empty' => t('No domains available.'),
  );
  $form['domain_list_pager'] = array('#theme' => 'pager');

  drupal_add_js(array('drd' => array('urlAjax' => url('drd/ajax/domain/'))), 'setting');

  return $form;
}

function drd_domain_build_row($d) {
  static $destination;
  static $charts;
  if (!isset($destination)) {
    $destination = drupal_get_destination();
  }
  if (!isset($charts)) {
    $charts = module_exists('chart');
  }

  $name = (strlen($d->name) > 35) ? substr($d->name, 0, 30) . '...' : $d->name;
  $warning = ($d->status_w) ? drd_icon('warning') : '';
  $error   = ($d->status_e) ? drd_icon('error') : '';
  $openstatus = drd_icon('more', 'triangle-1-s');
  $opencharts = ($charts) ? drd_icon('charts', 'triangle-1-s') : '';
  $edit = drd_icon('edit', 'pencil');
  $link = drd_icon('open', 'extlink');
  $count = db_select('drd_domain_alias', 'a')->fields('a', array('aid'))->condition('a.did', $d->did)->countQuery()->execute()->fetchField() - 1;
  $extra = '<div class="drd-info element-hidden">&nbsp;</div>';
  if ($count) {
    $count .= '<span class="drd-display">' . drd_icon('show', 'info');
    $count .= '<div class="detail">';

    $aliases = db_select('drd_domain_alias', 'a')->fields('a', array('url'))->condition('a.did', $d->did)->execute()->fetchAll();
    foreach ($aliases as $alias) {
      $count .= '<p>' . $alias->url . '</p>';
    }

    $count .= '</div>';
    $count .= '</span>';
  }

  return array(
    '#attributes' => array(
      'id' => 'drd-domain-' . $d->did,
      'class' => array('drd-data'),
    ),
    'name'   => array(
      'data' => l($name . $edit, 'admin/config/system/drd/domain/' . $d->did . '/edit', array('html' => TRUE, 'query' => $destination, 'attributes' => array('title' => $d->name))) . $extra,
      'class' => array('name'),
    ),
    'url'    => l($d->url . $link, drd_url($d), array('html' => TRUE, 'attributes' => array('target' => '_blank'))),
    'usessl' => ($d->usessl) ? t('Yes') : t('No'),
    'status' => array(
      'data' => $openstatus . (($d->status) ? t('Active') : t('Disabled')) . $warning . $error,
      'class' => array('pointer'),
    ),
    'inst'   => ($d->installed) ? t('Yes') : t('No'),
    'aliases'=> $count,
    'heartbeat' => array(
      'data' => ($d->installed && $d->status) ? $opencharts . '<div oid="' . $d->did . '" class="drd-heartbeat">&nbsp;</div>' : '',
      'class' => ($d->installed && $d->status && $charts) ? array('pointer', 'charts') : array(),
    ),
  );
}

/**
 * Builds the filter form for the domain list.
 *
 * @return type
 */
function drd_domain_filter_form($form, &$form_state) {
  if (!isset($_SESSION['drd']['filter']['domain']['sids'])) {
    $_SESSION['drd']['filter']['domain']['sids'] = array();
  }
  if (!isset($_SESSION['drd']['filter']['domain']['tags'])) {
    $_SESSION['drd']['filter']['domain']['tags'] = array();
  }
  if (!isset($_SESSION['drd']['filter']['domain']['drupalversions'])) {
    $_SESSION['drd']['filter']['domain']['drupalversions'] = array();
  }
  if (!isset($_SESSION['drd']['filter']['domain']['status'])) {
    $_SESSION['drd']['filter']['domain']['status'] = -1;
  }
  if (!isset($_SESSION['drd']['filter']['domain']['ssl'])) {
    $_SESSION['drd']['filter']['domain']['ssl'] = -1;
  }
  if (!isset($_SESSION['drd']['filter']['domain']['installed'])) {
    $_SESSION['drd']['filter']['domain']['installed'] = -1;
  }
  if (!isset($_SESSION['drd']['filter']['domain']['status_w'])) {
    $_SESSION['drd']['filter']['domain']['status_w'] = -1;
  }
  if (!isset($_SESSION['drd']['filter']['domain']['status_e'])) {
    $_SESSION['drd']['filter']['domain']['status_e'] = -1;
  }
  $form['filter_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['filter_wrapper']['filter_sids'] = array(
    '#type' => 'select',
    '#title' => t('Drupal core'),
    '#multiple' => TRUE,
    '#options' => drd_server_load_all(),
    '#default_value' => $_SESSION['drd']['filter']['domain']['sids'],
    '#size' => 10,
  );
  $form['filter_wrapper']['filter_status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#options' => array(
      '-1' => t('- any -'),
      '1' => t('Active'),
      '0' => t('Disabled'),
    ),
    '#default_value' => $_SESSION['drd']['filter']['domain']['status'],
  );
  $form['filter_wrapper']['filter_ssl'] = array(
    '#type' => 'select',
    '#title' => t('SSL'),
    '#options' => array(
      '-1' => t('- any -'),
      '1' => t('Yes'),
      '0' => t('No'),
    ),
    '#default_value' => $_SESSION['drd']['filter']['domain']['ssl'],
  );
  $form['filter_wrapper']['filter_installed'] = array(
    '#type' => 'select',
    '#title' => t('Installed'),
    '#options' => array(
      '-1' => t('- any -'),
      '1' => t('Yes'),
      '0' => t('No'),
    ),
    '#default_value' => $_SESSION['drd']['filter']['domain']['installed'],
  );
  $form['filter_wrapper']['filter_status_w'] = array(
    '#type' => 'select',
    '#title' => t('Warning Status'),
    '#options' => array(
      '-1' => t('- any -'),
      '1' => t('Yes'),
      '0' => t('No'),
    ),
    '#default_value' => $_SESSION['drd']['filter']['domain']['status_w'],
  );
  $form['filter_wrapper']['filter_status_e'] = array(
    '#type' => 'select',
    '#title' => t('Error Status'),
    '#options' => array(
      '-1' => t('- any -'),
      '1' => t('Yes'),
      '0' => t('No'),
    ),
    '#default_value' => $_SESSION['drd']['filter']['domain']['status_e'],
  );
  $form['filter_wrapper']['filter_drupalversions'] = array(
    '#type' => 'select',
    '#title' => t('Drupal Versions'),
    '#multiple' => TRUE,
    '#options' => drd_drupalversion_load_all(),
    '#default_value' => $_SESSION['drd']['filter']['domain']['drupalversions'],
  );

  _drd_tags_build_filter_form('domain', $form);

  $form['filter_wrapper']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Apply'),
  );
  $form['filter_wrapper']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset'),
  );
  return $form;
}

function drd_domain_filter_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['filter_limit']) || ((int)$form_state['values']['filter_limit'] < 10)) {
    form_set_error('filter_limit', t('The limit needs to be numeric and at least a value of 10.'));
  }
}

function drd_domain_filter_form_submit($form, $form_state) {
  $values = $form_state['values'];
  if ($_POST['op'] == t('Apply')) {
    $_SESSION['drd']['filter']['domain']['sids'] = $values['filter_sids'];
    $_SESSION['drd']['filter']['domain']['status'] = $values['filter_status'];
    $_SESSION['drd']['filter']['domain']['ssl'] = $values['filter_ssl'];
    $_SESSION['drd']['filter']['domain']['installed'] = $values['filter_installed'];
    $_SESSION['drd']['filter']['domain']['status_w'] = $values['filter_status_w'];
    $_SESSION['drd']['filter']['domain']['status_e'] = $values['filter_status_e'];
    $_SESSION['drd']['filter']['domain']['tags'] = $values['filter_tags'];
    $_SESSION['drd']['filter']['domain']['drupalversions'] = $values['filter_drupalversions'];
  }
}

/**
 * Builds the filter query for the domain list.
 *
 * @param type $query
 */
function drd_domain_build_filter_query(&$query) {
  if (!empty($_SESSION['drd']['filter']['domain']['sids'])) {
    $query->condition('d.sid', $_SESSION['drd']['filter']['domain']['sids']);
  }
  if ($_SESSION['drd']['filter']['domain']['status'] >= 0) {
    $query->condition('d.status', $_SESSION['drd']['filter']['domain']['status']);
  }
  if ($_SESSION['drd']['filter']['domain']['ssl'] >= 0) {
    $query->condition('d.usessl', $_SESSION['drd']['filter']['domain']['ssl']);
  }
  if ($_SESSION['drd']['filter']['domain']['installed'] >= 0) {
    $query->condition('d.installed', $_SESSION['drd']['filter']['domain']['installed']);
  }
  if ($_SESSION['drd']['filter']['domain']['status_w'] >= 0) {
    $query->condition('d.status_w', $_SESSION['drd']['filter']['domain']['status_w']);
  }
  if ($_SESSION['drd']['filter']['domain']['status_e'] >= 0) {
    $query->condition('d.status_e', $_SESSION['drd']['filter']['domain']['status_e']);
  }
  if (!empty($_SESSION['drd']['filter']['domain']['drupalversions'])) {
    $query->join('drd_server', 's', 'd.sid = s.sid');
    $query->condition('s.drupalversion', $_SESSION['drd']['filter']['domain']['drupalversions']);
  }
  if (!empty($_SESSION['drd']['filter']['domain']['tags'])) {
    $query->join('drd_term', 't', 'd.did = t.oid');
    $query->condition('t.mode', 'domain')
        ->condition('t.tid', $_SESSION['drd']['filter']['domain']['tags']);
  }
}
