(function ($) {

Drupal.drd = Drupal.drd || {};
Drupal.settings.drd = Drupal.settings.drd || {};

Drupal.behaviors.drd = {
  attach: function (context) {
    if (Drupal.settings.drd.urlAjax === undefined) {
      return;
    }
    if (Drupal.settings.drd.oids === undefined) {
      Drupal.settings.drd.heartbeat = false;
      Drupal.settings.drd.queueCount = 0;
      Drupal.settings.drd.queueCountRun = 0;
      Drupal.settings.drd.queueCountInfo = 0;
      Drupal.settings.drd.oids = [];
      $('.drd-heartbeat').each(function() {
        Drupal.settings.drd.oids.push($(this).attr('oid'));
      });
    }
    $('.drd-heartbeat-toggle:not(.drd-processed)')
      .addClass('drd-processed')
      .click(function() {
        if (Drupal.settings.drd.heartbeat) {
          Drupal.settings.drd.heartbeat = false;
          clearInterval(Drupal.settings.drd.queueHeartbeat);
          $('.drd-heartbeat-toggle').addClass('off');
          $('.drd-heartbeat').html('');
        }
        else {
          Drupal.settings.drd.heartbeat = true;
          Drupal.settings.drd.queueHeartbeat = setInterval(Drupal.drd.heartbeat, 10000);
          $('.drd-heartbeat-toggle').removeClass('off');
          Drupal.drd.heartbeat();
        }
      });
    $('tr.drd-data td.pointer:not(.drd-processed)')
      .addClass('drd-processed')
      .click(function() {Drupal.drd.toggleInfoPanel(this)});
    $('#drd-messages span').click(function() {
      $('#drd-messages .content').html('');
      $(this).hide();
    });

    $('#edit-options legend span').append('<span id="drd-queue-info-count">&nbsp;</span>');
    $('#edit-options').prepend('<div id="drd-queue-info">&nbsp;</div>');
    Drupal.settings.drd.queueInfo = setInterval(Drupal.drd.info, 1000);
    Drupal.settings.drd.queueRun = setInterval(Drupal.drd.queue, 5000);
    Drupal.settings.drd.queueHeartbeat = setInterval(Drupal.drd.heartbeat, 10000);
    Drupal.drd.info();
    Drupal.drd.queue();
    Drupal.drd.heartbeat();
  }
};

Drupal.drd.queue = function() {
  while (Drupal.settings.drd.queueCount < Drupal.settings.drd.parallelAjaxThreads) {
    Drupal.settings.drd.queueCount++;
    Drupal.settings.drd.queueCountRun++;
    $.ajax({
      url: Drupal.settings.drd.urlAjax + 'queue/run/' + Drupal.settings.drd.queueCountRun,
      processData: false,
      async: true,
      complete: function (response) {
        var data = Drupal.drd.parse(response.response);
        if (data.status == 'OK') {
          clearInterval(Drupal.settings.drd.queueRun);
          Drupal.settings.drd.queueCount = 5000;
        }
        else {
          Drupal.settings.drd.queueCount--;
        }
      }
    });
  }
};

Drupal.drd.info = function() {
  Drupal.settings.drd.queueCountInfo++;
  $.ajax({
    url: Drupal.settings.drd.urlAjax + 'queue/info/' + Drupal.settings.drd.queueCountInfo,
    processData: false,
    async: true,
    complete: function (response) {
      var data = Drupal.drd.parse(response.response);
      var content = '&nbsp;';
      if (data.count == 0) {
        clearInterval(Drupal.settings.drd.queueInfo);
      }
      else {
        content = '<div id="content">' + data.count + ' items in queue' + '</div>';
      }
      $('#drd-queue-info-count').html(content);
      for (var item in data.info) {
        if (typeof data.info[item] == 'string') {
          $('#drd-queue-info').prepend('<div>' + data.info[item] + '</div>');
        }
        else {
          $('#'+data.info[item].completed).html('OK');
        }

      }
    }
  });
};

Drupal.drd.heartbeat = function() {
  if (!Drupal.settings.drd.heartbeat) {
    return;
  }
  $.each(Drupal.settings.drd.oids, function(i, v) {
    $.ajax({
      url: Drupal.settings.drd.urlAjax + 'heartbeat/' + v,
      processData: false,
      async: true,
      complete: function (response) {
        var data = Drupal.drd.parse(response.response);
        $('.drd-heartbeat[oid="'+v+'"]').html(data.heartbeat);
      }
    });
  });
};

Drupal.drd.parse = function(response) {
  var data;
  if (typeof response == 'string') {
    data = $.parseJSON(response);
  }
  else {
    data = response;
  }
  if (data.messages !== undefined && data.messages !== '') {
    $('#drd-messages .content').prepend(data.messages);
    $('#drd-messages span').show();
  }
  if (data.refresh_id !== undefined && data.refresh_id !== '') {
    var tds = $('#' + data.refresh_id + ' td');
    var i = 1;
    for (var item in data.refresh_data) {
      var tx = '';
      if (typeof data.refresh_data[item] == 'string') {
        tx = data.refresh_data[item];
      }
      else {
        tx = data.refresh_data[item].data;
      }
      $(tds[i]).html(tx);
      i++;
    }
    $(tds[0]).addClass('drd-refreshed');
    window.setTimeout(Drupal.drd.clearRefresh, 5000);
  }
  return data;
};

Drupal.drd.clearRefresh = function() {
  var elements = $('td.drd-refreshed');
  if (elements.length > 0) {
    $(elements[0]).removeClass('drd-refreshed');
  }
};

Drupal.drd.toggleInfoPanel = function(el) {
  var container = $(el).closest('tr.drd-data');
  var $info = $('.drd-info', container);
  var $mode = 'info';
  var $param = '';
  var $checkMode = true;
  if ($(el).hasClass('charts')) {
    $mode = 'charts';
    if (el.hasAttribute('period')) {
      $param = '/'+$(el).attr('period');
      $checkMode = false;
    }
    else {
      $param = '/24h';
    }
  }
  if ($checkMode) {
    var $attr = $info.attr('drd-mode');
    if ($attr !== undefined) {
      if ($attr == $mode) {
        $info.hide();
        $mode = false;
      }
      $info.removeAttr('drd-mode');
    }
  }
  if ($mode !== false) {
    $info.attr('drd-mode', $mode);
    $info.html('<span class="wait-message">'+Drupal.t('Please wait...')+'</span>');
    var $id = $(container).attr('id');
    $.get(Drupal.settings.drd.urlAjax + $mode + '/' + $id + $param, {}, function(response) {
      var data = Drupal.drd.parse(response);
      $info.html(data.data);
      $('.drd-box-messages span', $info).click(function() {
        $(this.parentNode.parentNode).remove();
        $.get(Drupal.settings.drd.urlAjax + 'messages_seen/' + $id, {}, function(response) {
          var data = Drupal.drd.parse(response);
          return true;
        });
      });
      $('.charts .button', $info).click(function() {
        Drupal.drd.toggleInfoPanel(this);
      });
    });
    $info.show();
  }
};

})(jQuery);
