<?php

/**
 * Build the form to edit Drupal core properties.
 *
 * @param type $form
 * @param type $form_state
 * @param type $sid
 * @return type
 */
function drd_server_edit($form, &$form_state, $sid = 0) {
  if ($sid) {
    $server = (array)drd_server_load($sid);
  }
  else {
    $server = array(
      'sid' => 0,
      'name' => '',
      'url' => '',
      'usessl' => 0,
      'status' => 1,
      'remoteinstall' => 0,
      'weight' => 0,
      'aes_key' => '',
      'aes_cipher' => '',
      'aes_iv' => '',
      'aes_impl' => '',
      'drupalversion' => '',
    );
  }
  if (empty($server['aes_iv'])) {
    $aes_iv_org = variable_get("aes_encryption_iv", '');
    aes_make_iv(TRUE);
    $server['aes_iv'] = variable_get("aes_encryption_iv", '');
    variable_set("aes_encryption_iv", $aes_iv_org);
  }

  $form['#server'] = $server;

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#default_value' => $server['name'],
    '#max_length' => 255,
  );
  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#default_value' => $server['url'],
    '#max_length' => 255,
    '#disabled' => ($server['sid'] > 0),
    '#description' => t('The domain (and optionally path) to the root of your Drupal core installation, e.g. www.example.com or www.example.com/drupal'),
  );

  _drd_tags_build_form('server', $server['sid'], $form);

  $form['usessl'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use SSL'),
    '#default_value' => $server['usessl'],
  );
  $form['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#default_value' => $server['status'],
  );
  $form['remoteinstall'] = array(
    '#type' => 'checkbox',
    '#title' => t('Support remote installation'),
    '#default_value' => $server['remoteinstall'],
  );
  $form['remoteinstallinfo'] = array(
    '#type' => 'container',
    '#states' => array(
      'invisible' => array(
        'input[name="remoteinstall"]' => array('checked' => FALSE),
      ),
    ),
  );
  $form['remoteinstallinfo']['info'] = array(
    '#markup' => t('To support remote installation, go to your remote Drupal core, move the file drd_server.install.php.txt to the Drupal core root directory and rename it to drd_server.install.php.'),
    '#prefix' => '<div class="messages warning">',
    '#suffix' => '</div>',
  );
  $form += aes_config();
  unset($form['aes']['aes_convert']);
  unset($form['aes']['view_method']);
  unset($form['aes']['aes_key_storage_method']);
  unset($form['aes']['aes_key_path']);
  unset($form['aes']['submit']);
  $form['aes']['aes_implementation']['#default_value'] = $server['aes_impl'];
  $form['aes']['aes_cipher']['#default_value'] = $server['aes_cipher'];
  $form['aes']['aes_iv'] = array(
    '#type' => 'textfield',
    '#title' => 'AES IV',
    '#default_value' => $server['aes_iv'],
    '#disabled' => TRUE,
  );

  if ($server['sid']) {
    $subform = drd_operation_remote('drd.config.server.read', 'server', $server['sid']);
    if (!empty($subform) && is_array($subform)) {
      $form = array_merge($form, $subform);
    }
  }

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validates the edited Drupal core properties before submitting them.
 *
 * @param type $form
 * @param type $form_state
 */
function drd_server_edit_validate($form, &$form_state) {
  // Try to validate the URL
  $url = $form_state['values']['url'];
  $usessl = $form_state['values']['usessl'];
  $valid = TRUE;
  if (strpos($url, '://') !== FALSE) {
    form_set_error('url', t('The URL can not contain a the protocol, i.e. do not input http:// at the beginning of the URL.'));
    $valid = FALSE;
  }
  if (strpos($url, ' ') !== FALSE) {
    form_set_error('url', t('The URL can not contain space characters.'));
    $valid = FALSE;
  }
  if (strpos($url, '/') === 0) {
    form_set_error('url', t('The URL should not contain leading slashes.'));
    $valid = FALSE;
  }
  if (!empty($url) && substr($url, -1) == '/') {
    form_set_error('url', t('The URL should not contain trailing slashes.'));
    $valid = FALSE;
  }
  if (empty($url)) {
    form_set_error('url', t('The URL can not be empty.'));
    $valid = FALSE;
  }
  if ($valid) {
    $url = ($usessl ? 'https://' : 'http://') . $url;
    try {
      $response = drupal_http_request($url, array('max_redirects' => 0));
      if ($response->code == 302) {
        $target = $response->redirect_url;
        if (!empty($target) && substr($target, -1) == '/') {
          $target = substr($target, 0, -1);
        }
        form_set_error('url', t('Your URL %url redirects to %target, please adjust your settings accordingly.', array('%url' => $url, '%target' => $target)));
        $response->code = 200;
      }
      if ($response->code != 200) {
        $valid = FALSE;
      }
    }
    catch (Exception $e) {
      $valid = FALSE;
    }
    if (!$valid) {
      form_set_error('url', t('Failed to connect to the URL %url.', array('%url' => $url)));
    }
    else {
      // Check the remote installation support
      if ($form_state['values']['remoteinstall']) {
        $valid = TRUE;
        $url .= '/drd_server.install.php';
        try {
          $response = drupal_http_request($url, array('max_redirects' => 0));
          if ($response->code != 200) {
            $valid = FALSE;
          }
        }
        catch (Exception $e) {
          $valid = FALSE;
        }
        if (!$valid) {
          form_set_error('remoteinstall', t('Remote installation file drd_server.install.php not available on this domain.'));
        }
      }
    }
  }

  if (empty($form_state['values']['aes_key'])) {
    form_set_error('aes_key', t("You should define an encryption keys."));
  }
  else {
    if ($form_state['values']['aes_key'] !== $form_state['values']['aes_key_c']) {
      form_set_error('aes_key', t("The encryption keys didn't match."));
    }
  }
}

/**
 * Submits the edited Drupal core properties and installs the Drupal core if required.
 *
 * @param type $form
 * @param type $form_state
 */
function drd_server_edit_submit($form, &$form_state) {
  $changed = FALSE;
  $values = $form_state['values'];
  $server = $form['#server'];
  $server['name'] = $values['name'];
  $server['url'] = $values['url'];
  $server['usessl'] = $values['usessl'];
  $server['status'] = $values['status'];
  $server['remoteinstall'] = $values['remoteinstall'];
  $server['aes_impl'] = $values['aes_implementation'];
  $server['aes_cipher'] = $values['aes_cipher'];
  $server['aes_iv'] = $values['aes_iv'];
  if (!empty($form_state['values']['aes_key'])) {
    $changed = ($server['aes_key'] !== $values['aes_key']);
    $server['aes_key'] = $values['aes_key'];
  }
  $new = ($server['sid'] == 0);
  if (!$new) {
    db_update('drd_server')
      ->fields($server)
      ->condition('sid', $server['sid'])
      ->execute();
  }
  else {
    drupal_write_record('drd_server', $server);
    $changed = FALSE;
    if ($server['remoteinstall']) {
      $result = drd_install_drd_server('install server', 'server', $server['sid'], 0, FALSE);
    }
    else {
      $result = t('This Drupal core does not support remote installation of the drd_server module. If you want to support remote installation, go to your remote Drupal core, move the file drd_server.install.php.txt to the Drupal core root directory and rename it to drd_server.install.php! After that, change the setting here for this Drupal core and turn remote installation on.');
    }
    if (!$result) {
      drupal_set_message('Drupal core installation failed, please check manually.', 'error');
    }
    else if ($result == 'Either module DRD_SERVER or AES can not be enabled.') {
      drupal_set_message($result, 'error');
    }
    else {
      drupal_set_message($result);
      $query = drupal_get_destination();
      $query['goto'] = '1';
      $query['destination'] = 'admin/config/system/drd/dashboard';
      $query['drd-extras'] = '1';
      $link = l('here', 'admin/config/system/drd/server/' . $server['sid'] . '/pushaes', array('query' => $query));
      drupal_set_message('After that, click ' . $link . ' to push the AES keys to the Drupal core.');
    }
    $goto = 'admin/config/system/drd/dashboard';
  }

  _drd_tags_save('server', $server['sid'], $values['tags']);

  if ($changed) {
    drd_update_aes($server['sid']);
  }
  if (!$new) {
    drd_operation_remote('drd.config.server.save', 'server', $server['sid'], $values);
  }

  if (!empty($goto)) {
    drupal_goto($goto);
  }
}
